eval '(exit $?0)' && eval 'PERL=`which perl`; exec $PERL -wS $0 ${1+"$@"}'
    & eval 'setenv PERL `which perl5`; exec $PERL -wS $0 $argv:q'
    if 0;

### Local Variables: ***
### mode:perl ***
### comment-column:0 ***
### comment-start: "### " ***
### comment-end: "***" ***
### End: ***
#
# ****************DO NOT MOVE OR CHANGE LINES ABOVE THIS*********************
#
# The first set of lines runs perl from any shell. The second set of lines
# identifies the rest of the file as PERL for EMACS autoformatting.
# See end of copyright for more information.
#
#
# -------------------------------------------------------------------
#                                   X-BONE
#
#                          http://www.isi.edu/xbone
#                USC Information Sciences Institute (USC/ISI)
#                   Marina del Rey, California 90292, USA
#                          Copyright (c) 1998-2005
#
# -------------------------------------------------------------------
#
# Copyright (c) 1998-2005 by the University of Southern California.
# All rights reserved.
#
# Permission to use, copy, modify, and distribute this software and
# its documentation in source and binary forms for non-commercial
# purposes and without fee is hereby granted, provided that the above
# copyright notice appear in all copies and that both the copyright
# notice and this permission notice appear in supporting
# documentation, and that any documentation, advertising materials,
# and other materials related to such distribution and use acknowledge
# that the software was developed by the University of Southern
# California, Information Sciences Institute.  The name of the
# University may not be used to endorse or promote products derived
# from this software without specific prior written permission.
#
# THE UNIVERSITY OF SOUTHERN CALIFORNIA MAKES NO REPRESENTATIONS ABOUT
# THE SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.  THIS SOFTWARE IS
# PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#
# Other copyrights might apply to parts of this software and are so
# noted when applicable.
#
# -------------------------------------------------------------------

use Data::Dumper;
use Getopt::Long;

use strict vars; 

my @aln_col_names = ();                 # variables...

# ["MinRTT",  "range",     81,     200,      2,     150], 
# $table{quantize} = {
#                        MaxRTT => {
#                                    quantizetype => "range"
#                                    quantizetable => {
#                                          0 => {
#                                             quantizedval => 200
#                                             minval => 100
#                                             maxval => 300
#                                          }
#                                          1 => {...
#                                          }
#                                     }
#                                  }
my %aln_quantization_table = ();        # which variable, which value..

#
# aln_data_table => { 
#                     1 => {
#                            MaxRTT => 100, 
#                            MaxCwnd => 2562,
#                           ....
#                          }
#                     ....
#                     2 => {
#                           ....
#                          }
#                    ....
#                   };
#
my %aln_data_table = ();                # raw data
my %quantized_aln_data_table = ();      # quantized data...

sub read_quantization_file($) { 
    
    my $qfile = shift; 
    open(QUANTIZE, $qfile) || die ("Cannot open the table with quantization data");
    while(<QUANTIZE>){
	
	chomp; 
	next if (/^\s*\#/);
	next if (/^\s*$/);
	
	my @entries = split /\s+/; 
	
	my ($var, $qtype, $minval, $maxval, $enum, $qval) = @entries;
	
	$aln_quantization_table{$var}{quantizetype} = $qtype;
	$aln_quantization_table{$var}{quantizetable}{$enum}{quantizedval} = $qval;
	$aln_quantization_table{$var}{quantizetable}{$enum}{minval} = $minval;
        $aln_quantization_table{$var}{quantizetable}{$enum}{maxval} = $maxval;
	
    };
    close(QUANTIZE);

    #print Dumper(\%aln_quantization_table);

};

#
# Assumes first line has the column names...If the names
# and data are not matched, the script will bomb...
# 

sub read_aln_data($){
    
    my $dfile = shift; 
    open(DATA, $dfile) || die ("Cannot open the table with quantization data");
    my $firstline = <DATA>; 
    chomp($firstline); 
    @aln_col_names = split /\s+/, $firstline; 
    #print "Col Names = " . join (";", @aln_col_names) . "\n"; 
    my $counter = 0; 
    while(<DATA>){
	chomp;
	$counter++;
	my $line = $_; 

	my @entries = split /\s+/, $line; 
	foreach my $colname (@aln_col_names) { 
	    
	    #=> Patch the min and maxrtt if they are equal to 0.
	    my $val = shift @entries; 
	    if ($colname =~ /(minrtt|maxrtt)/i) {
		$val = 1 if ($val == 0); 
	    }; 

	    $aln_data_table{$counter}{$colname} = $val; 
	};
	
	#print "line = $line; counter = $counter\n"; 
	#print Dumper($aln_data_table{$counter}); 
    };
    
    close(DATA); 

}

# Description:
#     Lookup quantization ranges and compute the enumeration value
#     and the value after quantization corresponding to the 
#     enumeration value.
# Arguments:
#     -
# Returns:
#     -
# Exceptions:
#     -
sub quantize_var ($$) {
    
    my ($attr, $val) = @_; 
    my ($found, $quantizeenum, $quantizedval) = (0,0,0); 
    
    # defaults{quantize} = {
    #                        MaxRTT => { 
    #                                    quantizetype => "range"
    #                                    quantizetable => { 
    #                                          0 => {
    #                                             quantizedval => 200
    #                                             minval => 100
    #                                             maxval => 300
    #                                          }
    #                                          1 => {...
    #                                          }
    #                                     }
    #                                  }

    if (exists $aln_quantization_table{$attr}){
	my $qhash = $aln_quantization_table{$attr};
	my $qtable= $qhash->{quantizetable};
	my $qtype = $_ = $qhash->{quantizetype};
	
	#=> Depending on the type of quantization, we would have to
	#   transform the value...
        SWITCH: { 

	    /range/ and do { 
		# do nothing...
		last SWITCH; 
	    };
	    
	    # Use the range on the log to the base 10...
	    /log10/ and do { 
		if ($val > 0) { $val = log($val)/log(10); }
		last SWITCH; 
	    };
	    
	    do {
		die("Unknown quantization type...");
	    }
	}
	
	foreach my $enum (sort {$a <=> $b} keys %{$qtable}){
	    my $minval = $qtable->{$enum}{minval};
	    my $maxval = $qtable->{$enum}{maxval};

	    if (($minval <= $val) and ($maxval >= $val)){
		$quantizedval  = $qtable->{$enum}{quantizedval};
		$quantizeenum = $enum; 
		$found = 1; 
		last; 
	    }; 
	};
    };
    
    return ($found, $quantizeenum, $quantizedval); 
};

# Go through each variable and generate the corresponding 
# output after quantization...
# 
sub process_aln_data() {
    
    foreach my $num (keys %aln_data_table){
	my $entry = $aln_data_table{$num}; 
	# For each variable, replace the value by the quantized value..
	foreach my $var (keys %{$entry}){
	    my $val = $entry->{$var}; 
	    $quantized_aln_data_table{$num}{$var} = $val;

	    next if ($var =~ /(\#|RemAddr|StartTimeSec)/); 

	    my ($found, $enum, $qval) = quantize_var($var, $val); 
	    #print "[$num][$var] $val => $qval \n"; 
            if ($found) { 
	        $quantized_aln_data_table{$num}{$var} = $enum;
	        $quantized_aln_data_table{$num}{"quantized_" . $var} = $qval;
            } else { 
	        $quantized_aln_data_table{$num}{$var} = $val;
            }
	};
    };
};


sub print_aln_data($) {
    
    my $output = shift; 
    my $fh; 

    open($fh, ">", $output) || die ("Cannot open output file $output");
    
    # Header...
    foreach my $var (@aln_col_names){ print $fh "$var\t"; };
    print $fh "\n"; 
    
    # Lines...
    foreach my $num (keys %quantized_aln_data_table){
	my $entry = $quantized_aln_data_table{$num}; 
	foreach my $var (@aln_col_names){
	    print $fh $entry->{$var} . "\t"; 
	};
	print $fh "\n"; 
    };
    
    close($fh); 

};

#
# Generate one file for each of the variables...
#

sub print_quantization_comparison_data($) {
    
    my $compfile = shift; 
    
    return if (not defined $compfile); 
    
    # Header...
    foreach my $var (@aln_col_names){ 
	next if ($var =~ /\#/);

	my $file = "$compfile.$var.txt"; 
	open(OUT, ">", $file) || die ("Cannot open $file"); 
	
	print OUT "$var\tQuantized_$var\n";
	# Lines...
	foreach my $num (keys %quantized_aln_data_table){
	    
            # Print out the quantized value and not the enum
            # Check if the variable has a quantized version to 
            # begin with
	    my $val1 = $aln_data_table{$num}{$var}; 
	    my $val2 = $quantized_aln_data_table{$num}{"quantized_" . $var}; 
            if (not defined $val2) { 
	       $val2 = $quantized_aln_data_table{$num}{$var}; 
            };

	    print OUT "$val1\t$val2\n"; 
	};
	
	close(OUT); 
    };
};

#
# MAIN....
#

my %opt = (
	   "qfile" => "quantize_table.txt",
	   "input" => "aln_data.txt"
	   );


sub Usage() {
    
    print "quantize.pl [-h|-help] | \n";
    print "            [[-c|-comparison] <name>] \n";
    print "             [[-o|output] <name>]\n";
    print "             [[-q|qfile] <name>]\n";
    print "             [[-i|input] <name>]]\n";
    print "\n";
    print "           -h: This help message\n";
    print "           -o: Output file after quantization \n";
    print "           -q: File with quantization table\n"; 
    print "           -i: File with input data\n"; 
    print "           -c: Generate comparison tables \n";
    print "               with in files <name>.<varname>.txt\n"; 
};

GetOptions("h|help"          => \$opt{help},
	   "c|comparison=s"  => \$opt{comparison},
	   "o|output=s"      => \$opt{output},
	   "q|qfile=s"      => \$opt{qfile},
	   "i|input=s"      => \$opt{input},
	   ) || Usage();

Usage() if ($opt{help});

print Dumper(\%opt); 

if (! -e $opt{qfile}) { 
    print "Error! Input file $opt{qfile} does not exist\n"; 
    die; 
}; 

if (! -e $opt{input}) { 
    print "Error! Input file $opt{input} does not exist\n"; 
    die; 
}; 


#
# 
#

read_quantization_file($opt{qfile}); 
print_aln_data($opt{output}) if (defined $opt{output});
read_aln_data($opt{input}); 
process_aln_data(); 
print_aln_data($opt{output}) if (defined $opt{output});
print_quantization_comparison_data($opt{comparison}) 
    if (defined $opt{comparison});

exit;
