eval '(exit $?0)' && eval 'PERL=`which perl`; exec $PERL -wS $0 ${1+"$@"}'
    & eval 'setenv PERL `which perl5`; exec $PERL -wS $0 $argv:q'
    if 0;

### Local Variables: ***
### mode:perl ***
### comment-column:0 ***
### comment-start: "### " ***
### comment-end: "***" ***
### End: ***
#
# ****************DO NOT MOVE OR CHANGE LINES ABOVE THIS*********************
#
# The first set of lines runs perl from any shell. The second set of lines 
# identifies the rest of the file as PERL for EMACS autoformatting. 
# See end of copyright for more information.
#
# 
# -------------------------------------------------------------------
#                                   X-BONE
#
#                          http://www.isi.edu/xbone
#                USC Information Sciences Institute (USC/ISI)
#                   Marina del Rey, California 90292, USA
#                          Copyright (c) 1998-2005
# 
# -------------------------------------------------------------------
#
# Copyright (c) 1998-2005 by the University of Southern California.
# All rights reserved.
#
# Permission to use, copy, modify, and distribute this software and
# its documentation in source and binary forms for non-commercial
# purposes and without fee is hereby granted, provided that the above
# copyright notice appear in all copies and that both the copyright
# notice and this permission notice appear in supporting
# documentation, and that any documentation, advertising materials,
# and other materials related to such distribution and use acknowledge
# that the software was developed by the University of Southern
# California, Information Sciences Institute.  The name of the
# University may not be used to endorse or promote products derived
# from this software without specific prior written permission.
# 
# THE UNIVERSITY OF SOUTHERN CALIFORNIA MAKES NO REPRESENTATIONS ABOUT
# THE SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.  THIS SOFTWARE IS
# PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#
# Other copyrights might apply to parts of this software and are so
# noted when applicable.
#
# -------------------------------------------------------------------
#  $RCSfile: alndb.pl,v $
#
# $Revision: 1.17 $
#   $Author: pingali $
#     $Date: 2006/02/16 20:04:04 $
#    $State: Exp $
# ----------------------------------------------------------------------------

# The first lines start PERL on any system where perl is in the path.
# This is a modified version of the "start perl" script provided in
# the PERL man pages, which starts perl on the remainder of the file
# regardless of whether it is run under sh, csh, or perl.
#
# This version is modified to dynamically locate the perl path, rather
# than requiring it be hard-coded.

#
#=> Post process the alndb output into a form that Feili's code 
#   can handle. 
#

package ANNInput; 

use strict;

use Data::Dumper; 
use POSIX; 
use Statistics::Descriptive;

BEGIN {
    use strict;
    use sigtrap;
    use FindBin;

    my $ldir = $FindBin::RealBin;
    unshift @INC, $ldir;
    unshift @INC, "$ldir/Lib";

}


my %data = ();    # Overall data hash 
my %conf = ( 'database'  => 'data.txt',
	     'outputprefix' => 'stats',
	     'debug'     => 1, 
	    ); 


#=> README....
my $readme =<<END
************************************************** 
                 README 

Output values according to the format that Feili Hou
requested in May 5, 2006. The data corresponding to 
the output variables is the count for each numberation
value for the output variable. That is if MaxRTT, an
example variable, is observed to take enum value 0 
10 times, enum value 1 20 and enum value 2 30 times, 
the output looks like:
<condition variable values>\t#\t10\t20\t30


The format is as follows...

All comments begin with semicolon (;)

;Conditions\t#\tOutput
<num condition vars>	#	1
;Var1	Var2	...#	OutputVar
;Maximum values
Val1	Val2	...#	OutputVal
;
;
<val> <val>...	# <enum1-count> <enum2-count> <enum3-count>
<val> <val>...	# <enum1-count> <enum2-count> <enum3-count>
....

The following is the head of output file for MaxRTT variable
==================================================================

;ANN input for variable MaxRTT. See README.txt for 
;formatting information.
;
;Conditions	#	Output
4	#	1
;Maximum values
;RemAddress	time_Hour	time_Day	time_Month	#	MaxRTT
16777215	23	31	11	#	5
;
;
4197923	11	8	4	#	0	0	0	7	0	0	
14217023	22	29	3	#	0	1	0	0	0	0	
4381251	1	5	4	#	0	0	2	0	0	0	
8389340	16	23	3	#	0	0	0	17	0	0	

====================================================================

END
;

#
# Read all the lines from the data file and 
# ignore all the variables not included in the 
# conf list. 

sub ReadFile(){ 
    
    my $file = $conf{database}; 
    
    die("Cannot open file $file") unless (-e $file and -r $file);

    open FILE, $file || die "Cannot read database file $file"; 

    #=> Input format....
    # 
    #;conditions     #       outputs
    # 4       #       5
    #;
    #; Maximum values of the variables
    #;RemAddress     time_Hour       time_Day        time_Month      #       MinRTT  MaxRTT  MaxCwnd PktsIn  PktsOut
    #16777215        23      31      11      #       5       5       4       4       4
    #;
    
    #=> Skip all the comments...
    while(<FILE>){
	next unless (/^\;\s*condition/i);
	last;
    };
    
    #=> Read the condition and output list...
    my $line = <FILE>;
    chomp $line; 
    if ($line =~ /(\d+)\s*\#\s*(\d+)/){
	$data{summary}{conditions} = $1; 
	$data{summary}{outputs} = $2; 
    };

    #=> Skip all the comments...
    while(<FILE>){
	next unless (/^\;\s*maximum/i);
	last;
    };

    #=> Read the variable name line. 
    $line = <FILE>;
    chomp $line; 
    $line =~ s/^\;\s*//g;
    $line =~ s/\s+/:/g;
    if ($line =~ /(\S+):\#:(\S+)/){
	my @condnlist = split /:/, $1; 
	my @outputlist = split /:/, $2; 
	$data{variables}{conditions} = [ @condnlist ];
	$data{variables}{outputs} = [ @outputlist ]; 
    };
    
    #=> Read the maxvalue line...
    #16777215        23      31      11      #       5       5       4       4       4
    $line = <FILE>;
    chomp $line; 
    $line =~ s/\s+\#\s+/\t/;
    my @maxvalues = split /\s+/, $line; 
    foreach my $var (@{$data{variables}{conditions}},
		     @{$data{variables}{outputs}}){
	$data{maxvalues}{$var} = shift @maxvalues;
    };
    
    # Now read the rest of the lines....
    $data{values} = {}; 
    my $count = 0; 
    while (<FILE>){
	chomp; 
	next if (/^\;/);
	#print $_ . "\n";
	
	# Store the input lines that are in the form:
 	# 
	#4545820 20      28      3       #       2       4       2       3       2
	#
	# into the values hash like this...
	# 
	#    'values' => {
	#                '4381251:10:8:4' => {
	#                                    'PktsIn' => {
	#                                                 '4' => 0,
	#                                                 '1' => 1,
	#                                                 '3' => 0,
	#                                                 '0' => 0,
	#                                                 '2' => 0
	#                                                 },
	#                                    'MinRTT' => {
	#                                                 '4' => 0,
	#                                                 '1' => 0,
	#                                                 '3' => 0,
	#                                                 '0' => 0,
	#                                                 '2' => 0,
	#                                                 '5' => 0
	#


	my ($conditions, $output) = ($_, $_);
	$conditions =~ s/\s+\#.*$//;
	$output =~ s/^.*\#\s+//;
	my @outputvals = split /\s+/, $output; 
	
	#=> This will form the index...
	$conditions =~ s/\s+/:/g;
	
	#=> 
	foreach my $var (@{$data{variables}{outputs}}){
	    my $val = shift @outputvals;
	    
	    #=> initialize if doesnt exist
	    if (not exists $data{values}{$conditions} or 
		not exists $data{values}{$conditions}{$var}){
		for my $i (0 .. $data{maxvalues}{$var}){
		    $data{values}{$conditions}{$var}{$i} = 0;
		}
	    };
	    
	    $data{values}{$conditions}{$var}{$val}++;
	    #print "{$conditions}{$var}{$val} = " . 
	    #	$data{values}{$conditions}{$var}{$val} . "\n";
	    
	};
	
	$count++; 
	#last if ($count > 10); 
	
    }
    close(FILE); 

}; 

sub PrintTable() { 

    #print Dumper(\%data) if ($conf{debug});
    
    # Create the output directory if necessary.
    system "mkdir", "-p", $conf{outputprefix}; 

    my @outputvars = @{$data{variables}{outputs}};
    my @condnvars = @{$data{variables}{conditions}};


    my $path = $conf{outputprefix} . "/" . "README.txt";
    open README, ">$path" || die "Cannot output path $path"; 
    print README $readme; 
    close(README);
    
    my %openfiles = (); 
    foreach my $var (@outputvars){
	my $f; 
	my $path = $conf{outputprefix} . "/" . $var . ".txt";
	open $f, ">$path" || die "Cannot output path $path"; 
	$openfiles{$var} = $f; 
    };
    
    #=> Print the header in each file....
    foreach my $var (@outputvars){
	my $f = $openfiles{$var};
	
	print $f ";ANN input for variable $var. See README.txt for \n";
        print $f ";formatting information.\n;\n";
	print $f ";Conditions\t#\tOutput\n";
	print $f ($#condnvars+1) . "\t#\t1\n";
	print $f ";Maximum values\n;";
	foreach my $v (@condnvars) {
	    print $f $v ."\t";
	}
	print $f "\#\t$var\n";	
	foreach my $v (@condnvars) {
	    print $f $data{maxvalues}{$v} ."\t";
	}
	print $f "\#\t" . $data{maxvalues}{$var}. "\n";
	print $f ";\n;Data\n";
    };
    

    foreach my $condnline (sort keys %{$data{values}}){ 

	#    'values' => {
	#                '4381251:10:8:4' => {
	#                                    'PktsIn' => {
	#                                                 '4' => 0,
	#                                                 '1' => 1,
	#                                                 '3' => 0,
	#                                                 '0' => 0,
	#                                                 '2' => 0
	#                                                 },
	#                                    'MinRTT' => {
	#                                                 '4' => 0,
	#                                                 '1' => 0,
	#                                                 '3' => 0,
	#                                                 '0' => 0,
	#                                                 '2' => 0,
	#                                                 '5' => 0
	#

        # Get back the conditions line that needs to be printed out.
	# condnline =  '4381251:10:8:4'
	# line =  '4381251	10	8	4'
	my $line = $condnline;
	$line =~ s/:/\t/g;

	#print Dumper($data{values}{$condnline});
		
	foreach my $var (@outputvars){	
	    my $max = $data{maxvalues}{$var}; 
	    my $hash = $data{values}{$condnline}{$var};
	    my $file = $openfiles{$var};
	    
	    my $val = "";
	    for my $i (0 .. $max){
		$val .= $hash->{$i} . "\t"; 
	    }
	    chomp $val;

	    print $file "$line\t\#\t$val\n";
	    #print "File $var\t=>\t$line\t\#\t$val\n" if ($conf{debug});
	};
    }
    
    foreach my $var (@outputvars){
	close($openfiles{$var});
    };
    
}; 

sub Usage () { 
    print "ann-input-gen.pl [-h|-help\n\t|[[-d|-dump]" .
	  " [[-i|-input] <input-file>] ".
	  " [[-o|-output] <output-directory>]\n";
    exit;
};



eval { 
    
    my $done = 0; 

    
    while (1) { 

	$_ = shift @ARGV; 

	last if (not defined $_);

        SWITCH: {
	  
	  /^-(help|h)$/ && do {  
	      Usage();  
	      exit; 
	  }; 

	  /^-(input|i)$/ && do {  
	      $conf{database} = shift @ARGV;
	      next;
	  }; 

	  /^-(output|o)$/ && do {  
	      $conf{outputprefix} = shift @ARGV;
	      next;
	  }; 
	      
	  last SWITCH; 
        };

    };
    
    ReadFile(); 
    PrintTable(); 

}; # Eval...

if ($@){
    if ($@ =~ /(arg)/){
	print "Error in arguments!\n";
	die; 
    } else {
	print "Unknown error! $@\n";
	die; 
    }
}
1; 
