eval '(exit $?0)' && eval 'PERL=`which perl`; exec $PERL -wS $0 ${1+"$@"}'
    & eval 'setenv PERL `which perl5`; exec $PERL -wS $0 $argv:q'
    if 0;

### Local Variables: ***
### mode:perl ***
### comment-column:0 ***
### comment-start: "### " ***
### comment-end: "***" ***
### End: ***
#
# ****************DO NOT MOVE OR CHANGE LINES ABOVE THIS*********************
#
# The first set of lines runs perl from any shell. The second set of lines 
# identifies the rest of the file as PERL for EMACS autoformatting. 
# See end of copyright for more information.
#
# 
# -------------------------------------------------------------------
#                                   X-BONE
#
#                          http://www.isi.edu/xbone
#                USC Information Sciences Institute (USC/ISI)
#                   Marina del Rey, California 90292, USA
#                          Copyright (c) 1998-2005
# 
# -------------------------------------------------------------------
#
# Copyright (c) 1998-2005 by the University of Southern California.
# All rights reserved.
#
# Permission to use, copy, modify, and distribute this software and
# its documentation in source and binary forms for non-commercial
# purposes and without fee is hereby granted, provided that the above
# copyright notice appear in all copies and that both the copyright
# notice and this permission notice appear in supporting
# documentation, and that any documentation, advertising materials,
# and other materials related to such distribution and use acknowledge
# that the software was developed by the University of Southern
# California, Information Sciences Institute.  The name of the
# University may not be used to endorse or promote products derived
# from this software without specific prior written permission.
# 
# THE UNIVERSITY OF SOUTHERN CALIFORNIA MAKES NO REPRESENTATIONS ABOUT
# THE SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.  THIS SOFTWARE IS
# PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#
# Other copyrights might apply to parts of this software and are so
# noted when applicable.
#
# -------------------------------------------------------------------
#  $RCSfile: alndb.pl,v $
#
# $Revision: 1.17 $
#   $Author: pingali $
#     $Date: 2006/02/16 20:04:04 $
#    $State: Exp $
# ----------------------------------------------------------------------------

# The first lines start PERL on any system where perl is in the path.
# This is a modified version of the "start perl" script provided in
# the PERL man pages, which starts perl on the remainder of the file
# regardless of whether it is run under sh, csh, or perl.
#
# This version is modified to dynamically locate the perl path, rather
# than requiring it be hard-coded.


package Process; 

use strict;

use Data::Dumper; 
use POSIX; 
use Parse::AccessLogEntry;
use Statistics::Descriptive;

BEGIN {
    use strict;
    use sigtrap;
    use FindBin;

    my $ldir = $FindBin::RealBin;
    unshift @INC, $ldir;
    unshift @INC, "$ldir/Lib";

}

my %data = (); 

my %conf = ( 'database'  => 'access_log',
	     'outputprefix' => './',
             'statsfile'    => "log_stats.txt",
             'annstatsfile' => "ann_log_stats.txt",
	     'ann'       => 1,  # Input to ANN?
	     'debug'     => 1, 
	     'large'     => 20000, # how large is a large file? 
	     'variables' => {
		 "pic"  => "(jpg|gif|png|tif|ico|jpeg)",
		 "html" => "(html|htm|shtml|stm|cfm|xml)",
		 "pdf"  => "(pdf)",
		 "rss"  => "(rss)",
		 "css"  => "(css)",
		 "swf"  => "(swf)",
		 "script"  => "\\\.(cgi|php)",
		 "asp"  => "\\\.(asp)", 
		 "media"  => "\\\.(avi|mov|ra|smil|mp3|mpg|mpeg|mp4)", 
                 "java" => "\\\.(js|jar)",
		 "dynpage" => "([\&\?=\@])", 
		 "regpage" => "^([\&\?=\@]+\$)", 
	     },
	     'variableorder' => [qw (pic html pdf rss css swf script 
				    asp java media dynpage regpage )],
	    ); 


#############################################################
# Storage space for the summary....
#############################################################
my %stats = (
	     TotalCount => 0, 
 );

#
# Read all the lines from the data file and 
# ignore all the variables not included in the 
# conf list. 

sub ReadFile(){ 
    
    my $P=Parse::AccessLogEntry::new();
    my $prefix = $conf{database}; 
    my $count = 0; 

    my @suflist = qw (undef 1 2 3 4 5 6 7 8 9 10); 
    foreach my $suffix (@suflist){
	
	my $file = $prefix;
        $file .=  ".$suffix" if ($suffix ne "undef"); 
	next unless (-e $file); 
	
	open FILE, $file || die "Cannot read database file $file"; 
	
	while(<FILE>){
	    
	    # $Line is a string containing one line of an access log
	    my $hash = $P->parse($_);
	    
	    #print Dumper($hash); 
	    
	    if ( $hash->{bytes} =~ /^\d+$/ and 
		$hash->{bytes} > 0) {
		$data{all}{$count} = $hash; 
	    } 
	    
	    $count++; 
	    #last if ($count > 100);
	};
	
	close(FILE); 
    }; # foreach...

}; 

sub Summarize() { 

    foreach my $count (keys %{$data{all}}){
	my $hash = $data{all}{$count}; 
	
	#print Dumper($hash);

	my $file = $hash->{file};
	my $bytes = $hash->{bytes};
	
	my $site = "none";
	if ($file =~ /http:\/\/([^\/]+)\//i ){
	    $site = $1; 
	};

        my $type = "unknown";
	foreach my $t (@{$conf{variableorder}}){ 
	    my $pattern = $conf{variables}{$t}; 
	    if ($file =~ /$pattern/i){
		$type = $t;
		last;
	    };
	};
	#print "[$type] $site ===> $file\n" 
	#    if (($type =~ /(unknown|media)/) and 
	#	($site !~ /(mail.google.com)/i));

	push @{$data{sites}{$site}{all}{bytes}}, $bytes; 
	push @{$data{sites}{$site}{bytes}{$type}}, $bytes; 
	$data{sites}{$site}{types}{$type} = 1; 
	
	########################################
	# Compute information necessary for ANN
	########################################
        if ( $conf{ann} == 1) { 
	    $data{match}{$count}{site} = $site;
	    if ($type =~ /unknown/) {
		$data{match}{$count}{unknown} = 1;
	    } else {
		$data{match}{$count}{unknown} = 0;
	    };	    
	    foreach my $t (@{$conf{variableorder}}){ 
		my $pattern = $conf{variables}{$t}; 
		if ($file =~ /$pattern/i){
		    $data{match}{$count}{$t} = 1;
		} else {
		    $data{match}{$count}{$t} = 0;
	    };
	    };
	    if ($bytes > $conf{large}) {
		$data{match}{$count}{large} = 1; 
	    } else {
		$data{match}{$count}{large} = 0; 
	    };
	}
    };
    
    #print Dumper($data{match}); 
		 
    #print Dumper(\%{$data{sites}});
    
    foreach my $site (keys %{$data{sites}}){
	
	foreach my $type (keys %{$data{sites}{$site}{bytes}}){

	    my $stat = Statistics::Descriptive::Full->new();
	    my $arr = $data{sites}{$site}{bytes}{$type}; 
	    
	    $stat->add_data(@{$arr});
	
	    my $num      =$stat->count();
	    my $average  =$stat->mean();
	    my $variance =$stat->variance(); 
	    my $std_dev  =$stat->standard_deviation();
	    
	    $variance = 0 if (not defined $variance); 
	    $std_dev = 0 if (not defined $std_dev); 
	    
	    $average = int($average); 
	    $variance = int($variance);
	    $std_dev = int($std_dev); 
	    
	    my $normalization = $std_dev/$average; 
	    $normalization = int(100 * $normalization)/100; 
	    
	    $data{stats}{$site}{$type} = { 
		num => $num, 
		average => $average, 
		variance => $variance, 
		std_dev => $std_dev,
		normalization => $normalization, 
	    };

	}; 
    };

    
}; 

sub PrintTable { 
    
     do { 
        
        my $file = $conf{outputprefix} . $conf{statsfile};
        open (FILE, ">$file") || die "Cannot open $file"; 

	printf FILE "%18.18s", "Site"; 
	print FILE "\tType\tNos\tAvg\t\tStdDev\t\tNorm\n"; 
	
	foreach my $site (keys %{$data{stats}}){
	    foreach my $type (keys %{$data{stats}{$site}}){
		
		my $num = $data{stats}{$site}{$type}{num};
		my $average = $data{stats}{$site}{$type}{average};
		my $variance = $data{stats}{$site}{$type}{variance};
		my $std_dev = $data{stats}{$site}{$type}{std_dev};
		my $normalization = $data{stats}{$site}{$type}{normalization};
		
		printf FILE "%18.18s", $site; 
		print FILE "\t$type\t$num\t$average\t\t$std_dev\t\t$normalization\n";
	    };
	};
	
        close(FILE); 
     } while(0); 

     if ($conf{ann} == 1){   #unless...
	
        my $file = $conf{outputprefix} . $conf{annstatsfile};
        open (FILE, ">$file") || die "Cannot open $file"; 

	my @variables = ( @{$conf{variableorder}}, "unknown" );
	
        #=> Assign numbers to the sites...
        my %sitemap = (); 
        my $sitenum = 0; 
	foreach my $count (keys %{$data{match}}){
	    my $hash = $data{match}{$count}; 
	    my $site = $hash->{site}; 
            next if (defined $sitemap{$site});
            $sitemap{$site} = $sitenum++; 
        };

        #=> Print the header, sitemap and patterns 
        print FILE ";\n;******\n;Apache access logs processed to extract patterns\n"; 
        print FILE ";******\n;\n";
        print FILE ";\n;\n;SITEMAP\n";
        foreach my $site (keys %sitemap){
            #print FILE ";$site = $sitemap{$site}\n";
	    printf FILE ";%18.18s = %d\n", $site, $sitemap{$site}; 
        };
        
        print FILE ";\n;\n;PATTERN \n";
	foreach my $t (@{$conf{variableorder}}){ 
	    my $pattern = $conf{variables}{$t}; 
            print FILE ";$t => $pattern\n";
        }; 
	#=> Header...
	printf FILE ";\n;Data\n";
	printf FILE ";%.18s\t", "Site"; 
	foreach my $v (@variables){ 
	    printf FILE "%.5s\t", $v; 
	};


	print FILE "#\tLarge\n"; 
	foreach my $count (keys %{$data{match}}){
	    
	    my $hash = $data{match}{$count}; 
	    my $site = $hash->{site}; 
	    my $sitenum = $sitemap{$site}; 
    
	    printf FILE "%.18s\t", $sitenum; 
	    foreach my $v (@variables){ 
		print FILE $data{match}{$count}{$v} . "\t"; 
	    };
	    print FILE "#\t$hash->{large}\n"; 
	    
	};


        close(FILE); 
    };  

};


sub Usage () { 
    print "process_logs.pl [-h|-help]\n"; 
};



eval { 
    
    my $done = 0; 

    
    while (1) { 

	$_ = shift @ARGV; 

	last if (not defined $_);

        SWITCH: {
	  
	  /^-(help|h)$/ && do {  
	      Usage();  
	      exit; 
	  }; 
	      
	  last SWITCH; 
        };

    };
    
    #print Dumper(\%conf); 

    ReadFile(); 
    Summarize();
    PrintTable(); 
    
}; # Eval...

if ($@){
    if ($@ =~ /(arg)/){
	print "Error in arguments!\n";
	die; 
    } else {
	print "Unknown error! $@\n";
	die; 
    }
}
1; 
