eval '(exit $?0)' && eval 'PERL=`which perl`; exec $PERL -wS $0 ${1+"$@"}'
    & eval 'setenv PERL `which perl5`; exec $PERL -wS $0 $argv:q'
    if 0;

### Local Variables: ***
### mode:perl ***
### comment-column:0 ***
### comment-start: "### " ***
### comment-end: "***" ***
### End: ***
#
# ****************DO NOT MOVE OR CHANGE LINES ABOVE THIS*********************
#
# The first set of lines runs perl from any shell. The second set of lines 
# identifies the rest of the file as PERL for EMACS autoformatting. 
# See end of copyright for more information.
#
# 
# -------------------------------------------------------------------
#                                   X-BONE
#
#                          http://www.isi.edu/xbone
#                USC Information Sciences Institute (USC/ISI)
#                   Marina del Rey, California 90292, USA
#                          Copyright (c) 1998-2005
# 
# -------------------------------------------------------------------
#
# Copyright (c) 1998-2005 by the University of Southern California.
# All rights reserved.
#
# Permission to use, copy, modify, and distribute this software and
# its documentation in source and binary forms for non-commercial
# purposes and without fee is hereby granted, provided that the above
# copyright notice appear in all copies and that both the copyright
# notice and this permission notice appear in supporting
# documentation, and that any documentation, advertising materials,
# and other materials related to such distribution and use acknowledge
# that the software was developed by the University of Southern
# California, Information Sciences Institute.  The name of the
# University may not be used to endorse or promote products derived
# from this software without specific prior written permission.
# 
# THE UNIVERSITY OF SOUTHERN CALIFORNIA MAKES NO REPRESENTATIONS ABOUT
# THE SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.  THIS SOFTWARE IS
# PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#
# Other copyrights might apply to parts of this software and are so
# noted when applicable.
#
# -------------------------------------------------------------------
#  $RCSfile: alndb.pl,v $
#
# $Revision: 1.17 $
#   $Author: pingali $
#     $Date: 2006/02/16 20:04:04 $
#    $State: Exp $
# ----------------------------------------------------------------------------

# The first lines start PERL on any system where perl is in the path.
# This is a modified version of the "start perl" script provided in
# the PERL man pages, which starts perl on the remainder of the file
# regardless of whether it is run under sh, csh, or perl.
#
# This version is modified to dynamically locate the perl path, rather
# than requiring it be hard-coded.


package Process; 

use strict;

use Data::Dumper; 
use POSIX; 
use Statistics::Descriptive;

BEGIN {
    use strict;
    use sigtrap;
    use FindBin;

    my $ldir = $FindBin::RealBin;
    unshift @INC, $ldir;
    unshift @INC, "$ldir/Lib";

}

my %data = (); 

my %conf = ( 'database'  => 'data.txt',
	     'outputprefix' => 'stats/',
	     'debug'     => 1, 
	     'variables' => {
		              #StartTimeSec => 1,
			      Duration => 1,
			      RemAddress => 1, 
			      #PktsIn => 1, 
			      #PktsOut => 1, 
			      DataBytesIn => 1, 
			      DataBytesOut => 1, 
			      DataPktsIn => 1, 
			      DataPktsOut => 1, 
			      MaxCwnd => 1,
			      CurCwnd => 1,
			      MaxRTT => 1, 


                              # DataBytesIn is good enough
			      #approximation for the next two
			      #ThruBytesReceived => 1,
			      #ThruBytesAcked => 1,

	                    },	     

	     'cdf'       => { 
		              DataBytesIn => {
				  quantize => 10, 
				  units     => "Bytes(10s)",
			      },
			      MaxCwnd => {
				  quantize => 1, 
				  units     => "Bytes",
			      },
			      MaxRTT => {
				  quantize => 1, 
				  units     => "ms",

			      },
			      Duration => {
				  quantize => 1000, 
				  units     => "ms",

			      },
			      DataPktsIn => {
				  quantize => 1, 
				  units     => "Nos",
			      },
			      PktsIn => {
				  quantize => 1, 
				  units     => "Nos",
			      },
			      BW => {
				  quantize => 1, 
				  units     => "Bytes/sec",
			      },
			      UPPERBW => {
				  quantize => 1, 
				  units     => "Bytes/sec",
			      },
			      BENIFIT => {
				  quantize => 1, 
				  units     => "Ratio",
			      }
			    },

	    ); 


#############################################################
# Storage space for the summary....
#############################################################
my %stats = (
	     TotalCount => 0, 
	     Cwnd => {
		 DiffCount    => 0, # In how many cases did the max_cwnd diff
		                    # from cur_cwnd
		 CumDiffRatio => 0, # If there was a difference, 
		                    # how much was the difference?
		 
		 CumMaxCwnd   => 0, # cumulative sum of MaxCwnd
		 CumCurCwnd   => 0, # cumulative sum of Curcwnd
	     },
	     Time => {
		 CumDuration  => 0,
	     },
 );


#
# Read all the lines from the data file and 
# ignore all the variables not included in the 
# conf list. 

sub ReadFile(){ 
    
    my $file = $conf{database}; 
    my $variables_included = $conf{variables}; 
    my $cdf_variables = $conf{cdf}; 
   
    open FILE, $file || die "Cannot read database file $file"; 
    my @variables; 
    my $count = 0; 
    
    while(<FILE>){
	
	unless ($count){
	    
	    # First line. Read the header...
	    @variables = split /\s+/; 
	    $count++; 

	} else {
	    
	    # Read all the other lines...
	    
	    # First check if the data is ready to be
	    # processed. Ignore all the variables that 
	    # we care about which has 0 value! Nothing 
	    # should have zero value...
	    my $ignore = 0; 
	    my @vals = split /\s+/; 
	    my %entry = (); 
	    foreach my $var (@variables){
		my $val = shift @vals; 
		if (exists $variables_included->{$var}){ 
		    if (($var !~ /RemAddress/) and ($val == 0)){
			$ignore = 1; last; 
		    };
		    $entry{$var} = $val; 
		};
	    };		    
	    next if ($ignore); 
	    
	    #print "Entry = " . Dumper(\%entry) . "\n"; 

	    #
	    # now make the entries legit...
	    # 
	    foreach my $var (keys %entry){
		
		my $val = $entry{$var};		    
		if (exists $variables_included->{$var}){
		    $data{all}{$count}{$var} = $val; 
		};
		
		if (exists $cdf_variables->{$var}){
		    my $quantize = $cdf_variables->{$var}{quantize};
		    if (defined $quantize){ 
			$val = POSIX::ceil($val/$quantize);
		    }
		    $data{cdf}{$var}{$val}++; 
		};
	    };
	    
	    #
	    # Ok. Atleast variables are out of the way. Now
	    # compute the bandwidth...
	    
	    ############################################
	    # bandwidth
	    ############################################
	    my $bw = POSIX::ceil(($entry{DataBytesIn} * 1000000)/$entry{Duration});
	    my $quantize = $cdf_variables->{BW}{quantize};
	    if (defined $quantize){ 
		$bw = POSIX::ceil($bw/$quantize);
	    }
	    $data{cdf}{BW}{$bw}++;

	    ############################################
	    # minimum time
	    ############################################
	    my $numwindows = POSIX::ceil($entry{DataBytesIn}/$entry{CurCwnd});
	    $numwindows += 2; # One RTT for open, one RTT for close.
	    #print "numwindows = $numwindows, bytes = $entry{DataBytesIn}".
	    #      " MaxCwnd = $entry{MaxCwnd}\n"; 
	    my $time = $numwindows * $entry{MaxRTT}; # rtt is in ms
	    my $newbw = POSIX::ceil(($entry{DataBytesIn} * 1000)/$time);
	    $quantize = $cdf_variables->{UPPERBW}{quantize};
	    if (defined $quantize){ 
		$newbw = int($newbw/$quantize);
	    }
	    $data{cdf}{UPPERBW}{$newbw}++;
	    
	    ############################################
	    # Ratio
	    ############################################
	    my $ratio = int (($newbw/$bw) * 100)/100;
	    $data{cdf}{BENIFIT}{$ratio}++;

	    $data{all}{$count}{BW} = $bw; 
	    $data{all}{$count}{UPPERBW} = $newbw; 
	    $data{all}{$count}{BENIFIT} = $ratio; 
	    
	    push @{$data{correlation}{$entry{RemAddress}}}, $entry{DataBytesIn}; 
	    
	    $count++; 
	    #last if ($count > 100);
	};
    };
    
    close(FILE); 

}; 

sub PrintTable() { 

    print Dumper(\%data) if ($conf{debug});
    
}; 

sub DoCDFs() { 
    
    # Extract the bytes...
    foreach my $var (keys %{$data{cdf}}){

	my $vardata = $data{cdf}{$var}; 
        my $cum_total = 0;

	foreach my $val (sort {$a <=> $b} keys %{$vardata}){
	    my $num = $$vardata{$val};
            $cum_total += $val * $num; 
        };
      

	my $quantize = $conf{cdf}{$var}{quantize};
	$quantize = 1 if (not defined $quantize);

	# Foreach variable....
	my $prefix = $conf{outputprefix}; 
	my $file = $prefix . $var . ".txt"; 
	my $cumnum = 0; 
	my $cumval = 0; 
	my $count = 0; 
	my $num_total = $stats{TotalCount}; 
	my $units = $conf{cdf}{$var}{units};


	open FILE, ">$file" || die "Cannot open CDF file";
	#print FILE "CDF: $var (quantize = $quantize)\n";
	print FILE "Sr.No\t$units\tNum\tCumNum\t\%ile\tCumVal\t\%ile\n";
	foreach my $val (sort {$a <=> $b} keys %{$vardata}){
	    my $num = $$vardata{$val};

	    $cumnum += $num;
            $cumval += $num * $val;

	    my $percentile_num = int ((1/$num_total) * $cumnum * 10000)/100; 
	    my $percentile_val = int ((1/$cum_total) * $cumval * 10000)/100; 

	    print FILE "$count\t$val\t$num\t$cumnum\t$percentile_num\t$cumval\t$percentile_val\n";
	    $count++; 
	}
	
	close(FILE); 

    }; #foreach entry....

};
#
# Read all the lines from the data file and 
# ignore all the variables not included in the 
# conf list. 

sub PrintFile(){ 
    
    my $file = $conf{database}; 
    my $variables_included = $conf{variables}; 
    my $cdf_variables = $conf{cdf}; 
   
    open FILE, $file || die "Cannot read database file $file"; 
    my @variables; 
    my $count = 0; 
    
    while(<FILE>){
	
	unless ($count){
	    
	    # First line. Read the header...
	    @variables = split /\s+/; 
	    foreach my $var (@variables){
		if (exists $variables_included->{$var}){ 
		    print "$var\t"; 
		};
	    };		    
	    print "\n"; 
	    $count++; 

	} else {
	    
	    # Read all the other lines...
	    
	    # First check if the data is ready to be
	    # processed. Ignore all the variables that 
	    # we care about which has 0 value! Nothing 
	    # should have zero value...
	    my $ignore = 0; 
	    my @vals = split /\s+/; 
	    my %entry = (); 
	    foreach my $var (@variables){
		my $val = shift @vals; 
		if (exists $variables_included->{$var}){ 
		    print "$val\t"; 
		};
	    };
	    print "\n"; 
	    $count++; 
	    last if ($count > 100);
	};
    };
    close(FILE); 
}; 

sub PrintCorrelate(){
    
    #print Dumper($data{correlation});
    
    print "Addr\tNos\tAvg\tStdDev\tNorm\n"; 
    foreach my $addr (keys %{$data{correlation}}){ 
	
	my $arr = $data{correlation}{$addr}; 
	my $stat = Statistics::Descriptive::Full->new();
	
	#Adds data to the cached statistics values
	$stat->add_data(@{$arr});
	#
	my $num      =$stat->count();
	my $average  =$stat->mean();
	my $variance =$stat->variance(); 
	my $std_dev  =$stat->standard_deviation();
	
	$variance = 0 if (not defined $variance); 
	$std_dev = 0 if (not defined $std_dev); 
	
	$num = int($num); 
	$average = int($average); 
	$variance = int($average); 
	$std_dev = int($std_dev); 
	
	#print "#############\n$addr\n";
	#print "Num. of measurments = $num\n";
	#print "Average             = $average\n";
	#print "Variance            = $variance\n";
	#print "Standard Deviation  = $std_dev\n\n"; 
	
	# Ignore all the destinations that have less
	# than 1 entry. 
	next if ($num < 5); 

	my $normalization = $std_dev/$average; 
	#$normalization = int(1000 * $normalization)/100; 
	print "$addr\t$num\t$average\t$std_dev\t$normalization\n";

    };
}

#
# Summarize the data and print it out....
# 
sub SummarizeData() {

    
    #############################################################
    # Go over the all the data....
    #############################################################
    foreach my $num (sort { $a <=> $b } keys %{$data{all}}){
	
	$stats{TotalCount}++; 
	
	my $entry = $data{all}{$num}; 

	#Check the Cwnd
	if ($entry->{MaxCwnd} == 0 || $entry->{CurCwnd} == 0){
	    print Dumper($entry);
	}

	$stats{Cwnd}{CumCurCwnd} += $entry->{CurCwnd};
	$stats{Cwnd}{CumMaxCwnd} += $entry->{MaxCwnd};

	if ( $entry->{MaxCwnd} != $entry->{CurCwnd}) {
	    $stats{Cwnd}{DiffCount}++;
	    $stats{Cwnd}{CumDiffRatio} += $entry->{CurCwnd}/$entry->{MaxCwnd};
	};
	
	$stats{Time}{CumDuration} += $entry->{Duration}; 
	
    }; #foreach entry....

    
    #############################################################
    # Data......
    #############################################################
    
    # Print a standard readme 
    my $file = $conf{outputprefix} . "README.txt";
    open FILE, ">$file" || die "Cannot open file"; 
    print FILE "*_Summary.txt = Global statistics\n";  
    print FILE "*_<variable>.txt  = \n"; 
    print FILE "          Sr. No  = Sequence number of the value\n";
    print FILE "          <Units> = Value of the variable\n"; 
    print FILE "          Num     = Number of instances with that value\n"; 
    print FILE "          CumNum     = Cumulative number of instances\n"; 
    print FILE "          Percentile = Percentile of this value\n"; 
    print FILE "          CumVal     = Cumulative number of instances\n"; 
    print FILE "          Percentile = Percentile of this cumulative value\n"; 
    print FILE "\n\n";
    print FILE "For min time computation:\n";
    print FILE "        Time = RTT (1 for opening the connection)\n";
    print FILE "             + Ceiling(Data/CurCwnd) * RTT \n";
    print FILE "             + RTT (close)\n"; 
    print FILE " We might need a more complicated model\n"; 
    close(FILE); 
    
    # Print the summary. 
    $file = $conf{outputprefix} . "summary.txt";
    open FILE, ">$file" || die "Cannot open file"; 
    print FILE "Configuration" . Dumper(\%conf) . "\n"; 
    print FILE "Total number of entries => $stats{TotalCount}\n"; 
    
    #=> Cwnd....
    print FILE "Average MaxCwnd = " . $stats{Cwnd}{CumMaxCwnd}/$stats{TotalCount} . "\n";
    print FILE "Average CurCwnd = " . $stats{Cwnd}{CumCurCwnd}/$stats{TotalCount} . "\n";
    print FILE "Times when CurCwnd != MaxCwnd = " . $stats{Cwnd}{DiffCount} . "\n";

    my $t = $stats{Cwnd}{CumDiffRatio}/$stats{Cwnd}{DiffCount};
    $t = (int($t * 100))/100; # Round it off...
    print FILE "Average Cur/MaxCwnd in above cases = $t \n";
    
    #=> Time
    print FILE "Average Duration = " . int($stats{Time}{CumDuration}/$stats{TotalCount}) . "us \n";

    close(FILE); 
    
};


sub Usage () { 
	  print "process.pl [-h|-help\n\t|[[-d|-dump]" .
	      " [location|exit_stats|holidays|weather|strings|quantize|combined|processed]]\n".
	      "\t|[[-i|-initialize] [all|location|time|holidays|weather|strings|exit_stats|quantize]]\n". 
	      "\t|[[-u|-update] [all|location|time|weather|holidays|quantize|strings]]\n"; 
	  print "Details:\n"; 
	  print "        -h|-help: Print this message\n"; 
	  print "        -d|-dump: Dump the contents of the specified table\n"; 
	  print "                  processed = combined + quantization\n";
	  print "        -i|-initialize: Create the table specified (destroy\n".
	        "                    old table if necessary. \n"; 
	  print "        -u|-update: Update the location database with\n".
	        "                    information on recent destination IPs\n";
};



eval { 
    
    my $done = 0; 

    
    while (1) { 

	$_ = shift @ARGV; 

	last if (not defined $_);

        SWITCH: {
	  
	  /^-(help|h)$/ && do {  
	      Usage();  
	      exit; 
	  }; 
	      
	  last SWITCH; 
        };

    };
    
    #PrintFile(); 
    
    ReadFile(); 
    #PrintCorrelate(); 

    SummarizeData(); 
    DoCDFs(); 
    
}; # Eval...

if ($@){
    if ($@ =~ /(arg)/){
	print "Error in arguments!\n";
	die; 
    } else {
	print "Unknown error! $@\n";
	die; 
    }
}
1; 
