eval '(exit $?0)' && eval 'PERL=`which perl`; exec $PERL -wS $0 ${1+"$@"}'
    & eval 'setenv PERL `which perl5`; exec $PERL -wS $0 $argv:q'
    if 0;

### Local Variables: ***
### mode:perl ***
### comment-column:0 ***
### comment-start: "### " ***
### comment-end: "***" ***
### End: ***
#
# ****************DO NOT MOVE OR CHANGE LINES ABOVE THIS*********************
#
# The first set of lines runs perl from any shell. The second set of lines 
# identifies the rest of the file as PERL for EMACS autoformatting. 
# See end of copyright for more information.
#
# 
# -------------------------------------------------------------------
#                                   X-BONE
#
#                          http://www.isi.edu/xbone
#                USC Information Sciences Institute (USC/ISI)
#                   Marina del Rey, California 90292, USA
#                          Copyright (c) 1998-2005
# 
# -------------------------------------------------------------------
#
# Copyright (c) 1998-2005 by the University of Southern California.
# All rights reserved.
#
# Permission to use, copy, modify, and distribute this software and
# its documentation in source and binary forms for non-commercial
# purposes and without fee is hereby granted, provided that the above
# copyright notice appear in all copies and that both the copyright
# notice and this permission notice appear in supporting
# documentation, and that any documentation, advertising materials,
# and other materials related to such distribution and use acknowledge
# that the software was developed by the University of Southern
# California, Information Sciences Institute.  The name of the
# University may not be used to endorse or promote products derived
# from this software without specific prior written permission.
# 
# THE UNIVERSITY OF SOUTHERN CALIFORNIA MAKES NO REPRESENTATIONS ABOUT
# THE SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.  THIS SOFTWARE IS
# PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#
# Other copyrights might apply to parts of this software and are so
# noted when applicable.
#
# -------------------------------------------------------------------
#  $RCSfile: alndb.pl,v $
#
# $Revision: 1.17 $
#   $Author: pingali $
#     $Date: 2006/02/16 20:04:04 $
#    $State: Exp $
# ----------------------------------------------------------------------------

# The first lines start PERL on any system where perl is in the path.
# This is a modified version of the "start perl" script provided in
# the PERL man pages, which starts perl on the remainder of the file
# regardless of whether it is run under sh, csh, or perl.
#
# This version is modified to dynamically locate the perl path, rather
# than requiring it be hard-coded.


package Process; 

use strict;

use Data::Dumper; 

BEGIN {
    use strict;
    use sigtrap;
    use FindBin;

    my $ldir = $FindBin::RealBin;
    unshift @INC, $ldir;
    unshift @INC, "$ldir/Lib";

}

my %data = (); 

my %conf = ( 'database'  => 'data.txt',
	     'outputprefix' => 'stats/out_',
	     'debug'     => 1, 
	     'variables' => {
		              StartTimeSec => 1,
			      Duration => 1,
			      RemAddress => 1, 
			      PktsIn => 1, 
			      PktsOut => 1, 
			      DataBytesIn => 1, 
			      DataBytesOut => 1, 
			      DataPktsIn => 1, 
			      DataPktsOut => 1, 
			      DataOctetsIn => 1, 
			      DataOctetsOut => 1,
			      MaxCwnd => 1,
			      CurCwnd => 1,
			      MaxRTT => 1, 
	                    },	     

	     'cdf'       => { 
		              DataBytesIn => {
				  quantize => 10, 
			      },
			      MaxCwnd => {
				  quantize => 1, 
			      },
			      MaxRTT => {
				  quantize => 1, 
			      },
			      Duration => {
				  quantize => 1000, 
			      },
			      DataPktsIn => {
				  quantize => 1, 
			      }
			    },

	    ); 


#############################################################
# Storage space for the summary....
#############################################################
my %stats = (
	     TotalCount => 0, 
	     Cwnd => {
		 DiffCount    => 0, # In how many cases did the max_cwnd diff
		                    # from cur_cwnd
		 CumDiffRatio => 0, # If there was a difference, 
		                    # how much was the difference?
		 
		 CumMaxCwnd   => 0, # cumulative sum of MaxCwnd
		 CumCurCwnd   => 0, # cumulative sum of Curcwnd
	     },
	     Time => {
		 CumDuration  => 0,
	     },
 );


#
# Read all the lines from the data file and 
# ignore all the variables not included in the 
# conf list. 

sub ReadFile(){ 
    
    my $file = $conf{database}; 
    my $variables_included = $conf{variables}; 
    my $cdf_variables = $conf{cdf}; 
   
    open FILE, $file || die "Cannot read database file $file"; 
    my @variables; 
    my $count = 0; 
    
    while(<FILE>){
	
	unless ($count){
	    
	    # First line. Read the header...
	    @variables = split /\s+/; 
	    $count++; 

	} else {
	    
	    # Read all the other lines...
	    
	    # First check if the data is ready to be
	    # processed. Ignore all the variables that 
	    # we care about which has 0 value! Nothing 
	    # should have zero value...
	    my $ignore = 0; 
	    my @vals = split /\s+/; 
	    foreach my $var (@variables){
		my $val = shift @vals; 
		if (exists $variables_included->{$var}){ 
		    if (($var !~ /RemAddress/) and ($val == 0)){
			$ignore = 1; last; 
		    };
		};
	    };		    
	    next if ($ignore); 
	    
	    #
	    # now make the entries legit...
	    # 
	    @vals = split /\s+/; 
	    foreach my $var (@variables){
		
		my $val = shift @vals; 
		if (exists $variables_included->{$var}){
		    $data{all}{$count}{$var} = $val; 
		};
		
		if (exists $cdf_variables->{$var}){
		    my $quantize = $cdf_variables->{$var}{quantize};
		    if (defined $quantize){ 
			$val = int($val/$quantize) * $quantize;
		    }
		    $data{cdf}{$var}{$val}++; 
		};
	    };

	    $count++; 
	    #last if ($count > 10000);
	};
    };
    
    close(FILE); 

}; 

sub PrintTable() { 

    print Dumper(\%data) if ($conf{debug});
    
}; 

sub DoCDFs() { 
    
    # Extract the bytes...
    foreach my $var (keys %{$data{cdf}}){

	my $vardata = $data{cdf}{$var}; 

	my $quantize = $conf{cdf}{$var}{quantize};
	$quantize = 1 if (not defined $quantize);

	# Foreach variable....
	my $prefix = $conf{outputprefix}; 
	my $file = $prefix . $var . ".txt"; 
	my $cum = 0; 
	my $count = 0; 
	my $total = $stats{TotalCount}; 

	open FILE, ">$file" || die "Cannot open CDF file";
	#print FILE "CDF: $var (quantize = $quantize)\n";
	print FILE "Sr.No\tValue\tNum\tCum\tPercentile\n";
	foreach my $val (sort {$a <=> $b} keys %{$vardata}){
	    my $num = $$vardata{$val};
	    $cum += $num;
	    my $percentile = int ((1/$total) * $cum * 10000)/100; 
	    print FILE "$count\t$val\t$num\t$cum\t$percentile\n";
	    $count++; 
	}
	
	close(FILE); 

    }; #foreach entry....

};

#
# Summarize the data and print it out....
# 
sub SummarizeData() {

    
    #############################################################
    # Go over the all the data....
    #############################################################
    foreach my $num (sort { $a <=> $b } keys %{$data{all}}){
	
	$stats{TotalCount}++; 
	
	my $entry = $data{all}{$num}; 

	#Check the Cwnd
	if ($entry->{MaxCwnd} == 0 || $entry->{CurCwnd} == 0){
	    print Dumper($entry);
	}

	$stats{Cwnd}{CumCurCwnd} += $entry->{CurCwnd};
	$stats{Cwnd}{CumMaxCwnd} += $entry->{MaxCwnd};

	if ( $entry->{MaxCwnd} != $entry->{CurCwnd}) {
	    $stats{Cwnd}{DiffCount}++;
	    $stats{Cwnd}{CumDiffRatio} += $entry->{CurCwnd}/$entry->{MaxCwnd};
	};
	
	$stats{Time}{CumDuration} += $entry->{Duration}; 
	
    }; #foreach entry....

    
    #############################################################
    # Data......
    #############################################################
    
    # Print a standard readme 
    my $file = $conf{outputprefix} . "README.txt";
    open FILE, ">$file" || die "Cannot open file"; 
    print FILE "*_Summary.txt = Global statistics\n";  
    print FILE "*_<variable>.txt = \n"; 
    print FILE "          Sr. No = Sequence number of the value\n";
    print FILE "          Value  = Value of the variable\n"; 
    print FILE "          Num    = Number of instances with that value\n"; 
    print FILE "          Cum    = Cumulative number of instances\n"; 
    close(FILE); 
    
    # Print the summary. 
    $file = $conf{outputprefix} . "summary.txt";
    open FILE, ">$file" || die "Cannot open file"; 
    print FILE "Configuration" . Dumper(\%conf) . "\n"; 
    print FILE "Total number of entries => $stats{TotalCount}\n"; 
    
    #=> Cwnd....
    print FILE "Average MaxCwnd = " . $stats{Cwnd}{CumMaxCwnd}/$stats{TotalCount} . "\n";
    print FILE "Average CurCwnd = " . $stats{Cwnd}{CumCurCwnd}/$stats{TotalCount} . "\n";
    print FILE "Times when CurCwnd != MaxCwnd = " . $stats{Cwnd}{DiffCount} . "\n";

    my $t = $stats{Cwnd}{CumDiffRatio}/$stats{Cwnd}{DiffCount};
    $t = (int($t * 100))/100; # Round it off...
    print FILE "Average Cur/MaxCwnd in above cases = $t \n";
    
    #=> Time
    print FILE "Average Duration = " . int($stats{Time}{CumDuration}/$stats{TotalCount}) . "us \n";

    close(FILE); 
    
};


sub Usage () { 
	  print "process.pl [-h|-help\n\t|[[-d|-dump]" .
	      " [location|exit_stats|holidays|weather|strings|quantize|combined|processed]]\n".
	      "\t|[[-i|-initialize] [all|location|time|holidays|weather|strings|exit_stats|quantize]]\n". 
	      "\t|[[-u|-update] [all|location|time|weather|holidays|quantize|strings]]\n"; 
	  print "Details:\n"; 
	  print "        -h|-help: Print this message\n"; 
	  print "        -d|-dump: Dump the contents of the specified table\n"; 
	  print "                  processed = combined + quantization\n";
	  print "        -i|-initialize: Create the table specified (destroy\n".
	        "                    old table if necessary. \n"; 
	  print "        -u|-update: Update the location database with\n".
	        "                    information on recent destination IPs\n";
};



eval { 
    
    my $done = 0; 

    
    while (1) { 

	$_ = shift @ARGV; 

	last if (not defined $_);

        SWITCH: {
	  
	  /^-(help|h)$/ && do {  
	      Usage();  
	      exit; 
	  }; 
	      
	  last SWITCH; 
        };

    };
    
    ReadFile(); 
    SummarizeData(); 
    DoCDFs(); 
    
}; # Eval...

if ($@){
    if ($@ =~ /(arg)/){
	print "Error in arguments!\n";
	die; 
    } else {
	print "Unknown error! $@\n";
	die; 
    }
}
1; 
